/*
	Anunaki - beatvis.m
	by Leechbite

	- beat visualization scripts
*/

beatvisLoad() {
  mainbeatleft = mainnormal.findObject("main.beat.left");
  mainbeatright = mainnormal.findObject("main.beat.right");
  mainbeattrap = mainnormal.findObject("main.beat.trap");

  stickbeatleft = mainstick.findObject("stick.beat.left");
  stickbeatright = mainstick.findObject("stick.beat.right");
  stickbeattrapleft = mainstick.findObject("stick.beat.trap.left");
  stickbeattrapright = mainstick.findObject("stick.beat.trap.right");

  shadebeat = mainshade.findObject("shade.beat");
  shadebeattrap = mainshade.findObject("shade.beat.trap");

  fsbeatleft = mainfs.findObject("fs.beat.left");
  fsbeatright = mainfs.findObject("fs.beat.right");
  fsbeattrapleft = mainfs.findObject("fs.beat.trap.left");
  fsbeattrapright = mainfs.findObject("fs.beat.trap.right");

  deltaapha = 25;
  lastvuleft = 0; lastvuright=0;
  maxvu = 15;
  visupdating = FALSE;

  beatmode = getBeatMode();
  beatsource = getBeatSource();

  refreshBeat = new timer;
  refreshBeat.setDelay(50);
  
  if ((beatmode==BEAT_ON) || (beatmode==BEAT_REV))
    refreshBeat.start();
  if (beatmode==BEAT_STAY)
    beatvisSetAlpha(255,255);
}

beatvisUnLoad() {
  delete refreshBeat;
}

int getBeatMode() {
  if (lights_off_attrib.getData()=="1") return BEAT_OFF;
  if (lights_on_attrib.getData()=="1") return BEAT_STAY;
  if (lights_beat_attrib.getData()=="1") return BEAT_ON;
  if (lights_revbeat_attrib.getData()=="1") return BEAT_REV;
}

int getBeatSource() {
  if (lights_srcVU_attrib.getData()=="1") return BEAT_VU;
  if (lights_srcBass_attrib.getData()=="1") return BEAT_BASS;
  if (lights_srcTreble_attrib.getData()=="1") return BEAT_TREBLE;
}

// function called by attribs.m
lightsOnUpdate () {
  beatmode = getBeatMode();
  beatsource = getBeatSource();

  maxvu=15; // resets anim

  refreshBeat.stop();

  if (beatmode==BEAT_OFF) {
    beatvisSetAlpha(0, 0);
  } else if (beatmode==BEAT_STAY) {
    beatvisSetAlpha(255, 255);
  } else if ((beatmode==BEAT_ON) || (beatmode==BEAT_REV)) {
    refreshBeat.start();
  }
}

beatvisonTitleChange() {
  maxvu = 15;
}

beatvisSetAlpha(int left, int right) {
  mainbeatleft.setAlpha(left);
  mainbeatright.setAlpha(right);

  stickbeatleft.setAlpha(left);
  stickbeatright.setAlpha(right);

  shadebeat.setAlpha((left+right)/2);

  fsbeatleft.setAlpha(left);
  fsbeatright.setAlpha(right);

}

refreshBeat.onTimer() {
  int vuleft, vuright, x;

  if (visupdating) return;
  visupdating = TRUE;

  if (beatsource == BEAT_VU) {

    vuleft = getLeftVUMeter();
    vuright = getRightVUMeter();

  } else if (beatsource == BEAT_BASS) {

    vuleft = 0;
    vuright = 0;

    for (x=0; x<=4; x++)
    {
      vuleft = vuleft+getVisBand(0,x);
      vuright = vuright+getVisBand(0,x);
    }
    vuleft = vuleft/5;
    vuright = vuright/5;
  } else if (beatsource == BEAT_TREBLE) {

    vuleft = 0;
    vuright = 0;

    for (x=0; x<=5; x++)
    {
      vuleft = vuleft+getVisBand(0,75-x*2);
      vuright = vuright+getVisBand(0,75-x*2);
    }
    vuleft = vuleft/5;
    vuright = vuright/5;
  }

  if (vuleft>maxvu) maxvu = vuleft;
  if (vuright>maxvu) maxvu = vuright;

  vuleft = vuleft/maxvu*255;
  vuright = vuright/maxvu*255;

  int deltaleft = vuleft-lastvuleft;
  int deltaright = vuright-lastvuright;

  if ((deltaleft>0) && (deltaleft>deltaapha)) vuleft = lastvuleft + deltaapha;
  if ((deltaleft<0) && (deltaleft<(-deltaapha))) vuleft = lastvuleft - deltaapha;

  if ((deltaright>0) && (deltaright>deltaapha)) vuright = lastvuright + deltaapha;
  if ((deltaright<0) && (deltaright<(-deltaapha))) vuright = lastvuright - deltaapha;

  if (beatmode == BEAT_ON) {
    beatvisSetAlpha(vuleft, vuright);
  } else if (beatmode == BEAT_REV) {
    beatvisSetAlpha(255-vuleft, 255-vuright);
  }

  lastvuleft = vuleft;
  lastvuright = vuright;

  visupdating = FALSE;
}

mainbeatTrap.onRightButtonDown(int x, int y) {
  boolean disablemenu;

  beatvismenu = new popUpMenu;
  beatvismenu.addCommand("- Lights Control -", -1, 0, 1);
  beatvismenu.addSeparator();
  beatvismenu.addCommand("Off", 0, beatmode==BEAT_OFF, 0);
  beatvismenu.addCommand("Stay On", 1, beatmode==BEAT_STAY, 0);
  beatvismenu.addCommand("Beat (High CPU)", 2, beatmode==BEAT_ON, 0);
  beatvismenu.addCommand("Reversed Beat (High CPU)", 3, beatmode==BEAT_REV, 0);

  disablemenu = ((beatmode==BEAT_OFF) || (beatmode==BEAT_STAY));
  beatvismenu.addSeparator();
  beatvismenu.addCommand("- Beat Source -", -1, 0, 1);
  beatvismenu.addSeparator();

  beatvismenu.addCommand("VU Meter", 10, beatsource==BEAT_VU, disablemenu);
  beatvismenu.addCommand("Bass", 11, beatsource==BEAT_BASS, disablemenu);
  beatvismenu.addCommand("Treble", 12, beatsource==BEAT_TREBLE, disablemenu);

  int res = beatvismenu.popAtMouse();
  delete beatvismenu;

  if (res<0) return;

  if (res==0) lights_off_attrib.setData("1");
  if (res==1) lights_on_attrib.setData("1");
  if (res==2) lights_beat_attrib.setData("1");
  if (res==3) lights_revbeat_attrib.setData("1");

  if (res==10) lights_srcVU_attrib.setData("1");
  if (res==11) lights_srcBass_attrib.setData("1");
  if (res==12) lights_srcTreble_attrib.setData("1");

  complete;
}

mainbeatTrap.onLeftButtonUp(int x, int y) {
/* currently disabled option.

  if (beatmode==3) lights_off_attrib.setData("1");
  else if (beatmode==0) lights_on_attrib.setData("1");
  else if (beatmode==1) lights_beat_attrib.setData("1");
  else if (beatmode==2) lights_revbeat_attrib.setData("1");
*/

  return;

}

//  *** mouse redirection to main ;) ****
stickbeatTrapLeft.onRightButtonDown(int x, int y) { mainbeatTrap.onRightButtonDown(x, y); }
stickbeatTrapRight.onRightButtonDown(int x, int y) { mainbeatTrap.onRightButtonDown(x, y); }
stickbeatTrapLeft.onLeftButtonUp(int x, int y) { mainbeatTrap.onLeftButtonUp(x, y); }
stickbeatTrapRight.onLeftButtonUp(int x, int y) { mainbeatTrap.onLeftButtonUp(x, y); }

shadebeatTrap.onRightButtonDown(int x, int y) { mainbeatTrap.onRightButtonDown(x, y); }
shadebeatTrap.onLeftButtonUp(int x, int y) { mainbeatTrap.onLeftButtonUp(x, y); }

fsbeatTrapLeft.onRightButtonDown(int x, int y) { mainbeatTrap.onRightButtonDown(x, y); }
fsbeatTrapRight.onRightButtonDown(int x, int y) { mainbeatTrap.onRightButtonDown(x, y); }
fsbeatTrapLeft.onLeftButtonUp(int x, int y) { mainbeatTrap.onLeftButtonUp(x, y); }
fsbeatTrapRight.onLeftButtonUp(int x, int y) { mainbeatTrap.onLeftButtonUp(x, y); }

// **** attrib to call lightsonupdate
lights_off_attrib.onDataChanged() {  if (getData()=="1") lightsonupdate(); }
lights_on_attrib.onDataChanged() {  if (getData()=="1") lightsonupdate(); }
lights_beat_attrib.onDataChanged() {  if (getData()=="1") lightsonupdate(); }
lights_revbeat_attrib.onDataChanged() {  if (getData()=="1") lightsonupdate(); }
lights_srcVU_attrib.onDataChanged() {  if (getData()=="1") lightsonupdate(); }
lights_srcBass_attrib.onDataChanged() {  if (getData()=="1") lightsonupdate(); }
lights_srcTreble_attrib.onDataChanged() {  if (getData()=="1") lightsonupdate(); }
