/*************************************************************

  Anunaki - clock-analog.m
  by Leechbite

  - this script controls the analog-clock display.

*************************************************************/

#include <lib/std.mi>

function updatehands();

Global group frameGroup;
Global timer clockrefresh;
Global layer timehour, timemin, timesec;
Global float hour, min, sec, r1, r2, r3;
Global int lasttime;

System.onScriptLoaded() {

  frameGroup = getScriptGroup();

  timehour = frameGroup.findObject("clock.hour");
  timemin = frameGroup.findObject("clock.minute");
  timesec = frameGroup.findObject("clock.second");

  r1 = 0; r2 = 0; r3 = 0;
  timesec.fx_setRealTime(0);
  timesec.fx_setBilinear(1);
  timesec.fx_setGridSize(8,8);
  timesec.fx_setRect(0);
  timesec.fx_setSpeed(500);
  timesec.fx_setEnabled(1);
  timesec.fx_setLocalized(1);
  timesec.fx_setClear(0);

  timemin.fx_setRealTime(0);
  timemin.fx_setBilinear(1);
  timemin.fx_setGridSize(8,8);
  timemin.fx_setRect(0);
  timemin.fx_setSpeed(500);
  timemin.fx_setEnabled(1);
  timemin.fx_setLocalized(1);
  timemin.fx_setClear(0);

  timehour.fx_setRealTime(0);
  timehour.fx_setBilinear(1);
  timehour.fx_setGridSize(8,8);
  timehour.fx_setRect(0);
  timehour.fx_setSpeed(500);
  timehour.fx_setEnabled(1);
  timehour.fx_setLocalized(1);
  timehour.fx_setClear(0);

  clockrefresh = new timer;
  clockrefresh.setDelay(100);
  clockrefresh.onTimer();
}

system.onScriptUnloading() {
  frameGroup = NULL; // clears all events
  delete clockrefresh;
}

frameGroup.onSetVisible(boolean onoff) {
  if (onoff) {
    if (!clockrefresh.isRunning()) {
      clockrefresh.onTimer(); // freshes the clock display immediately
      clockrefresh.start();
    }
  } else {
    clockrefresh.stop(); // stop clock if not visible.
  }
}

timesec.fx_onGetPixelR(double r, double d, double x, double y) {
  return r-(sec*6.283184/60);
}

timemin.fx_onGetPixelR(double r, double d, double x, double y) {
  return r-(min*6.283184/60);
}

timehour.fx_onGetPixelR(double r, double d, double x, double y) {
  return r-(hour*6.283184/60);
}

updateHands() {
  timesec.fx_update();
  timemin.fx_update();
  timehour.fx_update();


  return;
}

clockrefresh.onTimer() {
  int timeint;
  string timetxt;

  timeint = getTimeOfDay();
  int diff = timeint-lasttime;
  if ((diff < 1000) && (diff > 0)) return; // last then one sec, dont bother updating.
  lasttime = timeint;

  if (timeint >= 43200000) {
    if (timeint >= 46800000)
      timeint = timeint -  43200000;
  } else if (timeint < 3600000) {
    timeint = timeint + 43200000;
  }

  timetxt = integerToLongTime(timeint);

  sec = stringToInteger(getToken(timetxt, ":", 2));
  min = stringToInteger(getToken(timetxt, ":", 1)) + sec/60;
  hour = stringToInteger(getToken(timetxt, ":", 0))*5 + min/12;

  updateHands();

}
