/*
	Anunaki - volseek.m
	by Leechbite

	- volume & wheel control scripts
*/

volseekLoad() {
  mainvolume = mainnormal.findObject("volume");
  mainvolumetrap = mainnormal.findObject("volume.trap");
  volslider = mainnormal.findObject("hidevol");
  balslider = mainnormal.findObject("hidebalance");

  mainvolumemap = new map;
  mainvolumemap.loadmap("main.volume.map");

  voladjust = 0;

  wheel = mainnormal.findObject("wheel");
  wheelgrove = mainnormal.findObject("wheel.grove");

  wheelmap = new map;
  wheelmap.loadmap("main.wheel.map");

  wheeladjust = 0;
  wheelmode = getWheelMode();
  if (wheelmode == WHEEL_VOL) {
    wheel.setXMLParam("tooltip", "Volume | Right-click to change wheel function");
  }
  if (wheelmode == WHEEL_SEEKER) {
    wheel.setXMLParam("tooltip", "Seek | Right-click to change wheel function");
  }
  if (wheelmode == WHEEL_BASS) {
    wheel.setXMLParam("tooltip", "Seek | Right-click to change wheel function");
  }
  if (wheelmode == WHEEL_TREBLE) {
    wheel.setXMLParam("tooltip", "Treble | Right-click to change wheel function");
  }
  if (wheelmode == WHEEL_BALANCE) {
    wheel.setXMLParam("tooltip", "Balance | Right-click to change wheel function");
  }

  stickvolume = mainstick.findObject("volume");
  stickvolumetrap = mainstick.findObject("volume.glareshadow");

  updatevol();
}

volseekUnLoad() {
  delete mainvolumemap;
  delete wheelmap;
}

// *********** volume scripts *****************

mainvolumetrap.onLeftButtonDown(int x, int y) {
  voladjust = 1;

  int newval = mainvolumemap.getValue(x-getLeft(), y-getTop());
  setVolume(newval);
}

mainvolumetrap.onMouseMove(int x, int y) {
  if (voladjust) {
    int newval = mainvolumemap.getValue(x-getLeft(), y-getTop());

    setVolume(newval);
  }
}

mainvolumetrap.onLeftButtonUp(int x, int y) {
  voladjust = 0;
}

stickvolumetrap.onLeftButtonDown(int x, int y) {
  voladjust = 1;

  int newval = (x-getLeft())*255/getWidth();
  if (newval < 0) newval = 0;
  if (newval > 255) newval = 255;
  setVolume(newval);
}

stickvolumetrap.onMouseMove(int x, int y) {
  if (voladjust) {
    int newval = (x-getLeft())*255/getWidth();

    if (newval < 0) newval = 0;
    if (newval > 255) newval = 255;

    setVolume(newval);
  }
}

stickvolumetrap.onLeftButtonUp(int x, int y) {
  voladjust = 0;
}

volslider.onPostedPosition(int newpos) {
  updatevol();
}

updatevol() {
  currvol = system.getVolume();

  int vol100 = currvol*100/255;

  setTempText("Volume: "+integerToString(vol100)+"%");
  string voltooltip = "Volume : "+integerToString(vol100)+"%";
  mainvolumetrap.setXMLParam("tooltip", voltooltip);
  stickvolumetrap.setXMLParam("tooltip", voltooltip);
  fsvolumeTrap.setXMLParam("tooltip", voltooltip);
  fsWheelVol.setXMLParam("tooltip", voltooltip);

  float frame = currvol*mainvolume.getLength()/255;
  int intframe = frame;
  if ((frame-intframe)>0.5) intframe++;
  if ((intframe ==0) && (currvol > 2)) intframe = 1;
  if (intframe>(mainvolume.getLength()-1)) intframe = mainvolume.getLength()-1;
  mainvolume.gotoFrame(intframe);

  frame = currvol*stickvolume.getLength()/255;
  intframe = frame;
  if ((frame-intframe)>0.5) intframe++;
  if ((intframe ==0) && (currvol > 2)) intframe = 1;
  if (intframe>(stickvolume.getLength()-1)) intframe = stickvolume.getLength()-1;
  stickvolume.gotoFrame(intframe);

  // *** fullscreen volume update (see fs-wheel.m and fs-wheel-declr.m)
  fsvolumeLed.setRegionFromMap(fsvolumeMap, currvol, 1);
}

// *********** treble/bass control scripts ***************
SetTreble(int val) {
  val -= 127;

  setEq(1);

  int move;
  for(int i=5; i<10; i++) {
    move = val*1.27/(10-i);
    setEqBand(i, move);
  }
}

SetBass(int val) {
  val -=127;

  setEq(1);

  int move;
  for(int i=0; i<5; i++) {
    move = val*1.27/(i+1);
    setEqBand(i, move);
  }
}

// ************** wheel control scripts *****************

int getWheelMode() {
  if (wheel_vol_attrib.getData()=="1") return WHEEL_VOL;
  if (wheel_bass_attrib.getData()=="1") return WHEEL_BASS;
  if (wheel_treble_attrib.getData()=="1") return WHEEL_TREBLE;
  if (wheel_seek_attrib.getData()=="1") return WHEEL_SEEKER;
  if (wheel_bal_attrib.getData()=="1") return WHEEL_BALANCE;
}

wheelOnUpdate () {
  wheelmode = getWheelMode();

  if (wheelmode == WHEEL_VOL) {
    wheel.setXMLParam("tooltip", "Volume | Right-click to change wheel function");
    setTempText("Wheel Mode: Volume");
  }
  if (wheelmode == WHEEL_SEEKER) {
    wheel.setXMLParam("tooltip", "Seek | Right-click to change wheel function");
    setTempText("Wheel Mode: Seeker");
  }
  if (wheelmode == WHEEL_BASS) {
    wheel.setXMLParam("tooltip", "Bass | Right-click to change wheel function");
    setTempText("Wheel Mode: Bass");
  }
  if (wheelmode == WHEEL_TREBLE) {
    wheel.setXMLParam("tooltip", "Treble | Right-click to change wheel function");
    setTempText("Wheel Mode: Treble");
  }
  if (wheelmode == WHEEL_BALANCE) {
    wheel.setXMLParam("tooltip", "Balance | Right-click to change wheel function");
    setTempText("Wheel Mode: Balance");
  }

}

wheel.onLeftButtonDown(int x, int y) {
  wheeladjust = 1;

  lastwheelval = wheelmap.getValue(x-getLeft(), y-getTop());

  lastvol = getVolume();
  lastpos = getPosition();
  lastbal = balslider.getPosition();
  lastbass = getPrivateInt("Anunaki", "Last Bass", 127);
  lasttreble = getPrivateInt("Anunaki", "Last Treble", 127);
}

wheel.onLeftButtonUp(int x, int y) {
  if (wheeladjust) {
    if (wheelmode == WHEEL_SEEKER) {
      seekto(lastpos);
    }
    if (wheelmode == WHEEL_BASS) {
      setBass(getPrivateInt("Anunaki", "Last Bass", 127));
    }
    if (wheelmode == WHEEL_TREBLE) {
      setTreble(getPrivateInt("Anunaki", "Last Treble", 127));
    }
  }
  wheeladjust = 0;
}

wheel.onMouseMove(int x, int y) {
  if (wheeladjust) {
    int rx = x-getLeft(); int ry = y-getTop();
    if ((rx<0) || (rx<0) || (rx>=getWidth()) || (ry>=getHeight()))
	return;

    int newwheelval = wheelmap.getValue(rx, ry);
    int wheelangle =newwheelval-lastwheelval;
    int adjust = wheelangle;

    if (wheelangle<0) wheelangle=255+wheelangle;

    wheelangle = wheelangle * 360/255;
    int frame = (wheelangle % 45)*wheelgrove.getLength()/44;

    wheelgrove.gotoFrame(frame);

    if (wheelmode == WHEEL_VOL) {
      boolean resetvol = 0;
      if (adjust > 200) {
        adjust = -(255-adjust+lastwheelval);
        resetvol = 1;
      }
      if (adjust < -200) {
        adjust = (255-lastwheelval)+(255+adjust);
        resetvol = 1;
      }

      int newvol = lastvol + adjust;
      if (newvol < 0) newvol = 0;
      if (newvol > 255) newvol = 255;

      setVolume(newvol);
      if ((adjust>50) || (adjust<-50) || (resetvol)) {
	lastvol = newvol;
	lastwheelval = newwheelval;
      }
    } else if (wheelmode == WHEEL_SEEKER) {
      boolean resetseeker = 0;
      if (adjust > 200) {
        adjust = -(255-adjust+lastwheelval);
        resetseeker = 1;
      }
      if (adjust < -200) {
        adjust = (255-lastwheelval)+(255+adjust);
        resetseeker = 1;
      }

      lastpos = lastpos + adjust*50;
      if (lastpos < 0) lastpos = 0;
      if (lastpos > getPlayItemLength()) lastpos =  getPlayItemLength();

      setTempText("Seek: "+System.integerToTime(lastpos));

      if ((adjust>50) || (adjust<-50) || (resetseeker)) {
	lastwheelval = newwheelval;
      }
    } else if (wheelmode == WHEEL_BALANCE) {
      boolean resetbalance = 0;
      if (adjust > 200) {
        adjust = -(255-adjust+lastwheelval);
        resetbalance = 1;
      }
      if (adjust < -200) {
        adjust = (255-lastwheelval)+(255+adjust);
        resetbalance = 1;
      }

      int newbal = lastbal + adjust;
      if (newbal < 0) newbal = 0;
      if (newbal > 255) newbal =  255;
      if ((newbal > 120) && (newbal < 133)) newbal = 127; // snapping to center code

      balslider.setPosition(newbal);
      int pr;
      pr = newbal-127;
      pr = pr*100/127;

      if (pr>0) {
	setTempText("Bal Right: "+integertostring(pr)+"%");
      } else if (pr<0) {
	setTempText("Bal Left: "+integertostring(-pr)+"%");
      } else {
	setTempText("Bal Center");
      }

      if ((adjust>50) || (adjust<-50) || (resetbalance)) {
	lastbal = newbal;
	lastwheelval = newwheelval;
      }
    } else if (wheelmode == WHEEL_BASS) {
      boolean resetbass = 0;
      if (adjust > 200) {
        adjust = -(255-adjust+lastwheelval);
        resetbass = 1;
      }
      if (adjust < -200) {
        adjust = (255-lastwheelval)+(255+adjust);
        resetbass = 1;
      }

      int newbass = lastbass + adjust;
      if (newbass < 0) newbass = 0;
      if (newbass > 255) newbass =  255;

      int pr;
      pr = newbass-127;
      pr = pr*100/127;

      setTempText("Bass: "+integertostring(pr)+"%");
      setPrivateInt("Anunaki", "Last Bass", newbass);
      //setBass(newbass);

      if ((adjust>50) || (adjust<-50) || (resetbass)) {
        lastbass = newbass;
	lastwheelval = newwheelval;
      }
    } else if (wheelmode == WHEEL_TREBLE) {
      boolean resettreble = 0;
      if (adjust > 200) {
        adjust = -(255-adjust+lastwheelval);
        resettreble = 1;
      }
      if (adjust < -200) {
        adjust = (255-lastwheelval)+(255+adjust);
        resettreble = 1;
      }

      int newtreble = lasttreble + adjust;
      if (newtreble < 0) newtreble = 0;
      if (newtreble > 255) newtreble =  255;

      int pr;
      pr = newtreble-127;
      pr = pr*100/127;

      setTempText("Treble: "+integertostring(pr)+"%");
      setPrivateInt("Anunaki", "Last Treble", newtreble);

      if ((adjust>50) || (adjust<-50) || (resettreble)) {
        lasttreble = newtreble;
	lastwheelval = newwheelval;
      }
    }
  }
}

wheel.onRightButtonDown(int x, int y)  {
  popupMenu wheelMenu;

  wheelMenu = new popupMenu;
  wheelMenu.addCommand("- Wheel Control -", -1, 0, 1);
  wheelMenu.addSeparator();
  wheelMenu.addCommand(" Volume", WHEEL_VOL, wheelmode==WHEEL_VOL, 0);
  wheelMenu.addCommand(" Seeker", WHEEL_SEEKER, wheelmode==WHEEL_SEEKER, 0);
  wheelMenu.addCommand(" Bass", WHEEL_BASS, wheelmode==WHEEL_BASS, 0);
  wheelMenu.addCommand(" Treble", WHEEL_TREBLE, wheelmode==WHEEL_TREBLE, 0);
  wheelMenu.addCommand(" Balance", WHEEL_BALANCE, wheelmode==WHEEL_BALANCE, 0);

  setXMLParam("tooltip", "");

  int res = wheelMenu.popAtMouse();
  delete wheelMenu;

  if (res<0) return;

  if (res == WHEEL_VOL) wheel_vol_attrib.setData("1");
  if (res == WHEEL_SEEKER) wheel_seek_attrib.setData("1");
  if (res == WHEEL_BASS) wheel_bass_attrib.setData("1");
  if (res == WHEEL_TREBLE) wheel_treble_attrib.setData("1");
  if (res == WHEEL_BALANCE) wheel_bal_attrib.setData("1");

  complete; 
}

wheel_vol_attrib.onDataChanged() {  if (getData()=="1") wheelonupdate(); }
wheel_seek_attrib.onDataChanged() {  if (getData()=="1") wheelonupdate(); }
wheel_bass_attrib.onDataChanged() {  if (getData()=="1") wheelonupdate(); }
wheel_treble_attrib.onDataChanged() {  if (getData()=="1") wheelonupdate(); }
wheel_bal_attrib.onDataChanged() {  if (getData()=="1") wheelonupdate(); }